#ifdef WIN32
#include <windows.h>
#else
#include <pthread.h>
#include <signal.h>
#endif
#include "thread.h"

namespace Msp {

struct Thread::Private
{
#ifdef WIN32
	HANDLE handle;
#else
	pthread_t handle;
#endif

	Private(): handle(0) { }

#ifdef WIN32
	static DWORD WINAPI main_wrapper(void *t)
	{ reinterpret_cast<Thread *>(t)->main(); return 0; }
#else
	static void *main_wrapper(void *t)
	{ reinterpret_cast<Thread *>(t)->main(); return 0; }
#endif
};


Thread::Thread():
	priv_(new Private),
	launched_(false)
{ }

Thread::~Thread()
{
	if(launched_)
	{
		kill();
		join();
	}
	delete priv_;
}

void Thread::join()
{
	if(!launched_)
		return;

#ifdef WIN32
	WaitForSingleObject(priv_->handle, INFINITE);
#else
	pthread_join(priv_->handle, 0);
#endif
	launched_ = false;
}

void Thread::kill()
{
#ifdef WIN32
	TerminateThread(priv_->handle, 0);
#else
	pthread_kill(priv_->handle, SIGKILL);
#endif
}

void Thread::launch()
{
	if(launched_)
		return;

#ifdef WIN32
	DWORD dummy;  // Win9x needs the lpTthreadId parameter
	priv_->handle = CreateThread(0, 0, &Private::main_wrapper, this, 0, &dummy);
#else
	pthread_create(&priv_->handle, 0, &Private::main_wrapper, this);
#endif
	launched_ = true;
}

} // namespace Msp
